@@ -29,6 +29,7 @@
 #include <gnome.h>
 
 #include "gst.h"
+#include "gst-hig-dialog.h"
 #include "callbacks.h"
 #include "boot-settings.h"
 #include "table.h"
@@ -42,58 +43,58 @@ extern GtkWidget *boot_table;
 gchar *initrd_label_str = N_("_Initrd image path:");
 gchar *module_label_str = N_("_Module path:");
 
-static GList *
-settings_dev_list (GstBootImageType ctype)
+static void
+settings_dev_list (GtkComboBox *combo, GstBootImageType ctype)
 {
-	GList *list = NULL;
-
-	xmlNodePtr  root;
-	xmlNodePtr  part;
-	xmlNodePtr  node;
-
-	gchar *dev;
-	gchar *type;
+	xmlNodePtr    root;
+	xmlNodePtr    part;
+	xmlNodePtr    node;
+	gchar        *type;
+	gchar        *dev;
+	GtkTreeModel *model;
 
 	root = gst_xml_doc_get_root (tool->config);
 	part = gst_xml_element_find_first (root, "partitions");
+
+	model = gtk_combo_box_get_model (combo);
+	gtk_list_store_clear (GTK_LIST_STORE (model));
+
 	for (node = gst_xml_element_find_first (part, "partition");
 	     node; node = gst_xml_element_find_next (node, "partition"))
 	{
 		dev = gst_xml_get_child_content (node, "dev");
 		type = gst_xml_get_child_content (node, "type");
 		if (ctype == label_to_type (type))
-			list = g_list_append (list, g_strdup (dev));
+			gtk_combo_box_append_text (combo, dev);
 	}
-
-	return (list);
 }
 
 static void
 on_type_entry_change (GtkWidget *w, gpointer data)
 {
-	BootSettingsGui *gui;
-	const gchar *buf;
-	GstBootImageType type;
-	BootImage *image;
+	BootSettingsGui  *gui;
+	GstBootImageType  type;
+	BootImage        *image;
+	const gchar      *buf;
 
 	gui = (BootSettingsGui *) data;
 
-	buf = gtk_entry_get_text (GTK_ENTRY (gui->type->entry));
+	buf = boot_settings_get_type (gui);
 	
 	if (strlen (buf) > 0)
 	{
 		type = label_to_type (buf);
 		if ((type == TYPE_LINUX) || (type == TYPE_HURD))
 		{
-			gtk_combo_set_popdown_strings (gui->root, settings_dev_list (type));
+			settings_dev_list (GTK_COMBO_BOX (gui->root), type);
 			image = gui->image;
+
 			if (image->root)
-				gst_ui_entry_set_text (GTK_ENTRY (gui->root->entry), image->root);
+				gst_ui_entry_set_text (GTK_ENTRY (GTK_BIN (gui->root)->child), image->root);
 		}
 		else if (type != TYPE_LINSWAP)
-			gtk_combo_set_popdown_strings (gui->device, settings_dev_list (type));
+			settings_dev_list (GTK_COMBO_BOX (gui->device), type);
 	}	
-	return;
 }
 
 static void
@@ -120,13 +121,13 @@ boot_settings_gui_new (BootImage *image, GtkWidget *parent)
 	/* Basic frame */
 	gui->basic_frame = glade_xml_get_widget (gui->xml, "settings_basic_frame");
 	gui->name = GTK_ENTRY (glade_xml_get_widget (gui->xml, "settings_name"));
-	gui->type = GTK_COMBO (glade_xml_get_widget (gui->xml, "settings_type"));
+	gui->type = glade_xml_get_widget (gui->xml, "settings_type");
 
 	/* Image frame */
 	gui->image_frame = glade_xml_get_widget (gui->xml, "settings_image_frame");
 	gui->image_widget = glade_xml_get_widget (gui->xml, "settings_image");
 	gui->image_entry = GTK_ENTRY (glade_xml_get_widget (gui->xml, "settings_image_entry"));	
-	gui->root = GTK_COMBO (glade_xml_get_widget (gui->xml, "settings_root_combo"));
+	gui->root = glade_xml_get_widget (gui->xml, "settings_root");
 	gui->initrd_label = glade_xml_get_widget (gui->xml, "settings_initrd_label");
 	gui->initrd_widget = glade_xml_get_widget (gui->xml, "settings_initrd");
 	gui->initrd_entry = GTK_ENTRY (glade_xml_get_widget (gui->xml, "settings_initrd_entry"));
@@ -135,7 +136,7 @@ boot_settings_gui_new (BootImage *image, GtkWidget *parent)
 
 	/* Other frame */
 	gui->other_frame = glade_xml_get_widget (gui->xml, "settings_other_frame");
-	gui->device = GTK_COMBO (glade_xml_get_widget (gui->xml, "settings_device"));
+	gui->device = glade_xml_get_widget (gui->xml, "settings_device");
 
 	/* Security frame */
 	gui->use_password = GTK_CHECK_BUTTON (glade_xml_get_widget (gui->xml, "settings_use_password"));
@@ -144,22 +145,15 @@ boot_settings_gui_new (BootImage *image, GtkWidget *parent)
 	gui->password = GTK_ENTRY (glade_xml_get_widget (gui->xml, "settings_password"));
 	gui->password_confirm = GTK_ENTRY (glade_xml_get_widget (gui->xml, "settings_password_confirm"));
 
+	boot_settings_fill_type_list (gui);
+
 	/* Connect signals */
-	g_signal_connect (G_OBJECT (gui->name), "activate",
-			  G_CALLBACK (gui_grab_focus), (gpointer) gui->type->entry);
-	g_signal_connect (G_OBJECT (gui->type->entry), "changed",
+	g_signal_connect (G_OBJECT (gui->type), "changed",
 			  G_CALLBACK (on_type_entry_change), (gpointer) gui);
-	g_signal_connect (G_OBJECT (gui->image_entry), "activate",
-			  G_CALLBACK (gui_grab_focus), (gpointer) gui->root);
-	g_signal_connect (G_OBJECT (gui->root->entry), "activate",
-			  G_CALLBACK (gui_grab_focus), (gpointer) gui->initrd_widget);
-	g_signal_connect (G_OBJECT (gui->initrd_entry), "activate",
-			  G_CALLBACK (gui_grab_focus), (gpointer) gui->append);
 	g_signal_connect (G_OBJECT (gui->append_browse), "clicked",
 	                  G_CALLBACK (on_boot_append_browse_clicked), (gpointer) gui);
 	g_signal_connect (G_OBJECT (gui->use_password), "clicked",
 			  G_CALLBACK (on_use_password_clicked), (gpointer) gui);
-
 	return gui;
 }
 
@@ -195,9 +189,11 @@ compare (gconstpointer a, gconstpointer b)
 	return (GINT_TO_POINTER(strcmp (a,b)));
 }
 
-GList *
-settings_type_list (void)
+void
+boot_settings_fill_type_list (BootSettingsGui *gui)
 {
+	GtkWidget    *combo;
+	GtkTreeModel *model;
 	GList *list = NULL;
 
 	xmlNodePtr  root;
@@ -206,22 +202,71 @@ settings_type_list (void)
 
 	gchar *dev;
 	gchar *type;
+
+	combo = gui->type;
+	model = gtk_combo_box_get_model (GTK_COMBO_BOX (combo));
+	gtk_list_store_clear (GTK_LIST_STORE (model));
+
+	root  = gst_xml_doc_get_root (tool->config);
+	part  = gst_xml_element_find_first (root, "partitions");
 	
-	root = gst_xml_doc_get_root (tool->config);
-	part = gst_xml_element_find_first (root, "partitions");
 	for (node = gst_xml_element_find_first (part, "partition");
 	     node; node = gst_xml_element_find_next (node, "partition"))
 	{
 		dev = gst_xml_get_child_content (node, "dev");
 		type = type_to_label (label_to_type (gst_xml_get_child_content (node, "type")));
-		if ((g_list_find_custom (list, type, (GCompareFunc)compare) == NULL) &&
-		    (label_to_type (type) != TYPE_LINSWAP))
-			list = g_list_append (list, g_strdup (type));
+		if ((g_list_find_custom (list, type, (GCompareFunc)compare) == NULL)
+		    && (label_to_type (type) != TYPE_LINSWAP))
+			gtk_combo_box_append_text (GTK_COMBO_BOX (combo), type);
+			list = g_list_append (list, type);
 	}
 
-	return (list);
+	g_list_free (list);
 }
 
+const gchar*
+boot_settings_get_type (BootSettingsGui *gui)
+{
+	GtkComboBox  *combo;
+	GtkTreeModel *model;
+	GtkTreeIter   iter;
+	gchar        *type;
+
+	combo = GTK_COMBO_BOX (gui->type);
+	model = gtk_combo_box_get_model (combo);
+
+	if (gtk_combo_box_get_active_iter (combo, &iter)) {
+		gtk_tree_model_get (model, &iter, 0, &type, -1);
+		return type;
+	}
+
+	return NULL;
+}
+
+void
+boot_settings_set_type (BootSettingsGui *gui, gchar *type)
+{
+	GtkComboBox  *combo;
+	GtkTreeModel *model;
+	GtkTreeIter   iter;
+	gboolean      valid;
+	gchar        *t;
+
+	combo = GTK_COMBO_BOX (gui->type);
+	model = gtk_combo_box_get_model (combo);
+
+	valid = gtk_tree_model_get_iter_first (model, &iter);
+
+	while (valid) {
+		gtk_tree_model_get (model, &iter, 0, &t, -1);
+
+		if (strcmp (t, type) == 0) {
+			gtk_combo_box_set_active_iter (combo, &iter);
+			valid = FALSE;
+		} else
+			valid = gtk_tree_model_iter_next (model, &iter);
+	}
+}
 
 void
 boot_settings_gui_setup (BootSettingsGui *gui, GtkWidget *top)
@@ -232,9 +277,10 @@ boot_settings_gui_setup (BootSettingsGui *gui, GtkWidget *top)
 
 	if (image->type == TYPE_LINUX)
 	{
-		gtk_combo_set_popdown_strings (gui->root, settings_dev_list (image->type));
+		settings_dev_list (GTK_COMBO_BOX (gui->root), image->type);
+
 		if (image->root)
-			gst_ui_entry_set_text (GTK_ENTRY (gui->root->entry), image->root);
+			gst_ui_entry_set_text (GTK_ENTRY (GTK_BIN (gui->root)->child), image->root);
 
 		if (error = boot_image_valid_initrd (image))
 			gst_ui_entry_set_text (gui->initrd_entry, "");
@@ -242,12 +288,14 @@ boot_settings_gui_setup (BootSettingsGui *gui, GtkWidget *top)
 			gst_ui_entry_set_text (gui->initrd_entry, image->initrd);
 
 		gst_ui_entry_set_text (gui->append, image->append);
+		boot_settings_set_type (gui, type_to_label (image->type));
 	}
 	else if (image->type == TYPE_HURD)
 	{
-		gtk_combo_set_popdown_strings (gui->root, settings_dev_list (image->type));
+		settings_dev_list (GTK_COMBO_BOX (gui->root), image->type);
+
 		if (image->root)
-			gst_ui_entry_set_text (GTK_ENTRY (gui->root->entry), image->root);
+			gst_ui_entry_set_text (GTK_ENTRY (GTK_BIN (gui->root)->child), image->root);
 
 		if (error = boot_image_valid_module (image))
 			gst_ui_entry_set_text (gui->initrd_entry, "");
@@ -255,15 +303,16 @@ boot_settings_gui_setup (BootSettingsGui *gui, GtkWidget *top)
 			gst_ui_entry_set_text (gui->initrd_entry, image->module);
 
 		gst_ui_entry_set_text (gui->append, image->append);
+		boot_settings_set_type (gui, type_to_label (image->type));
 	}
 	else
 	{
-		gtk_combo_set_popdown_strings (gui->device, settings_dev_list (image->type));
+		settings_dev_list (GTK_COMBO_BOX (gui->device), image->type);
+
 		if (image->image)
-		  gst_ui_entry_set_text (GTK_ENTRY (gui->device->entry), image->image);
+			gst_ui_entry_set_text (GTK_ENTRY (GTK_BIN (gui->device)->child), image->image);
 	}
 	
-	gst_ui_entry_set_text (gui->type->entry, type_to_label (image->type));
 	gst_ui_entry_set_text (gui->name, image->label);
 
 	if (error = boot_image_valid_root (image))
@@ -271,7 +320,7 @@ boot_settings_gui_setup (BootSettingsGui *gui, GtkWidget *top)
 	else
 		gst_ui_entry_set_text (gui->image_entry, image->image);
 
-	gst_ui_entry_set_text (gui->device->entry, image->image);
+	gst_ui_entry_set_text (GTK_BIN (gui->device)->child, image->image);
 
 	if (image->password != NULL)
 	{
@@ -315,8 +364,9 @@ boot_settings_gui_save (BootSettingsGui *gui, gboolean check)
 	if (image->append)   g_free (image->append);
 	if (image->password) g_free (image->password);
 
-	image->type = label_to_type (gtk_entry_get_text (GTK_ENTRY (gui->type->entry)));
+	image->type = label_to_type (boot_settings_get_type (gui));
 	image->label = g_strdup (gtk_entry_get_text (gui->name));
+
 	if (gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (gui->use_password)))
 		image->password = g_strdup (gtk_entry_get_text (gui->password));
 	else
@@ -324,14 +374,14 @@ boot_settings_gui_save (BootSettingsGui *gui, gboolean check)
 
 	if (image->type == TYPE_LINUX)
 	{
-		image->root = g_strdup (gtk_entry_get_text (GTK_ENTRY (gui->root->entry)));
+		image->root = g_strdup (gtk_entry_get_text (GTK_ENTRY (GTK_BIN (gui->root)->child)));
 		image->append = g_strdup (gtk_entry_get_text (gui->append));
 		image->image = g_strdup (gtk_entry_get_text (gui->image_entry));
 		image->initrd = g_strdup (gtk_entry_get_text (gui->initrd_entry));
 	}
 	else if (image->type == TYPE_HURD)
 	{
-		image->root = g_strdup (gtk_entry_get_text (GTK_ENTRY (gui->root->entry)));
+		image->root = g_strdup (gtk_entry_get_text (GTK_ENTRY (GTK_BIN (gui->root)->child)));
 		image->append = g_strdup (gtk_entry_get_text (gui->append));
 		image->image = g_strdup (gtk_entry_get_text (gui->image_entry));
 		image->module = g_strdup (gtk_entry_get_text (gui->initrd_entry));
@@ -340,25 +390,21 @@ boot_settings_gui_save (BootSettingsGui *gui, gboolean check)
 	{
 		image->root = g_strdup ("");
 		image->append = g_strdup ("");
-		image->image = g_strdup (gtk_entry_get_text (GTK_ENTRY (gui->device->entry)));
+		image->image = g_strdup (gtk_entry_get_text (GTK_ENTRY (GTK_BIN (gui->device)->child)));
 		image->initrd = g_strdup ("");
 	}
 
-	if (check)
-	{
+	if (check) {
 		msg_error = boot_image_check (image);
 
-		if (msg_error)
-		{
-			boot_settings_gui_error (GTK_WINDOW (gui->top), msg_error);
-			return FALSE;
-		}
+		if ((!msg_error) && (strcmp (gtk_entry_get_text (gui->password),
+					     gtk_entry_get_text (gui->password_confirm)) != 0))
+			msg_error = g_strdup (_("Password confirmation is not correct"));
 
-		if (strcmp (gtk_entry_get_text (gui->password),
-			    gtk_entry_get_text (gui->password_confirm)) != 0)
-		{
-			msg_error = g_strdup (_("Password confirmation isn't correct."));
+		if (msg_error) {
 			boot_settings_gui_error (GTK_WINDOW (gui->top), msg_error);
+			g_free (msg_error);
+
 			return FALSE;
 		}
 	}
@@ -374,10 +420,13 @@ boot_settings_gui_error (GtkWindow *parent, gchar *error)
 {
 	GtkWidget *d;
 
-	d = gtk_message_dialog_new (parent, GTK_DIALOG_MODAL | GTK_DIALOG_DESTROY_WITH_PARENT,
-				    GTK_MESSAGE_ERROR, GTK_BUTTONS_OK, error);
-
-	g_free (error);
+	d = gst_hig_dialog_new (parent,
+				GTK_DIALOG_MODAL | GTK_DIALOG_DESTROY_WITH_PARENT,
+				GST_HIG_MESSAGE_ERROR,
+				_("Error modifying boot image"),
+				error,
+				GTK_STOCK_CLOSE, GTK_RESPONSE_CLOSE,
+				NULL);
 	gtk_dialog_run (GTK_DIALOG (d));
 	gtk_widget_destroy (d);
 }
