@@ -286,14 +286,20 @@ filter_editable (GtkEditable *editable, const gchar *text, gint length,
 		 gint *pos, gpointer data)
 {
 	int i = 0;
-	gchar *s = NULL;
+	gchar *pre, *post, *s = NULL;
 	gboolean success;
 	gboolean string_ok = TRUE;
 	EditableFilterRules rules = GPOINTER_TO_INT (data);
 
 	if (rules & EF_ALLOW_IP) {
-		s = g_strconcat (gtk_editable_get_chars (editable, 0, -1), text, NULL);
+		pre  = gtk_editable_get_chars (editable, 0, *pos);
+		post = gtk_editable_get_chars (editable, *pos, -1);
+
+		s = g_strconcat (pre, text, post, NULL);
 		success = is_ip_text_ok (s);
+
+		g_free (pre);
+		g_free (post);
 		g_free (s);
 	} else {
 		while ((i < length) && (string_ok)) {
@@ -882,6 +888,12 @@ on_connection_cancel_clicked (GtkWidget *w, gpointer data)
 	gtk_widget_hide (window);
 }
 
+void
+on_connection_help_clicked (GtkWidget *w, gpointer data)
+{
+	gst_tool_show_help (tool, "tool-modify-connection");
+}
+
 void
 on_connection_modified (GtkWidget *w, gpointer data)
 {
@@ -1301,6 +1313,12 @@ on_network_druid_hide (GtkWidget *w, gpointer data)
 	return TRUE;
 }
 
+void
+on_network_druid_help (GtkWidget *w, gpointer data)
+{
+	gst_tool_show_help (tool, "tool-add-new-interface");
+}
+
 gboolean
 on_network_druid_page_back (GnomeDruidPage *druid_page, GnomeDruid *druid, gpointer data)
 {
@@ -1440,7 +1458,7 @@ on_network_profiles_button_clicked (GtkWidget *widget, gpointer data)
 	GtkWidget *profiles_dialog = gst_dialog_get_widget (tool->main_dialog,
 							    "network_profiles_dialog");
 	
-	gtk_dialog_run (GTK_DIALOG (profiles_dialog));
+	while (gtk_dialog_run (GTK_DIALOG (profiles_dialog)) != GTK_RESPONSE_CLOSE);
 	gtk_widget_hide (profiles_dialog);
 
 	profile_populate_option_menu (tool, root);
@@ -1477,6 +1495,12 @@ on_network_profile_new_clicked (GtkWidget *widget, gpointer data)
 	gtk_entry_set_text (GTK_ENTRY (profile_description), "");
 }
 
+void
+on_network_profile_help_clicked (GtkWidget *widget, gpointer data)
+{
+	gst_tool_show_help (tool, "tool-add-new-profile");
+}
+
 void
 on_network_profile_table_selection_changed (GtkWidget *widget, gpointer data)
 {
